#include "osHandles.h"              // Includes all OS Files

#include "System/cpu.h"             // Initialize CPU Hardware
//initialize drivers
#include "drivers/watchdog.h"
#include "drivers/uart0.h"       	// Initialize UART

#include "utils/rprintf.h"			// Reduced printf.  STDIO uses lot of FLASH space & Stack space
#include "fat/diskio.h"

#include "tasks/userInterface.hpp"	// User interface functions to interact through UART
#include "tasks/SD_card.hpp"		// SD card tasks to interact with User
#include "tasks/mp3.hpp"
#include "tasks/buttonMonitor.hpp"
#include "drivers/TSOP1236.h"

#include "utils/timer.hpp"
#include "utils/vector.hpp"
#include "CString.hpp"

void FatFsDiskTimer(void* p)
{
	disk_timerproc();
}

/* INTERRUPT VECTORS:
 * 0:    OS Timer Tick
 * 1:    TSOP
 * 2:    UART0 Interrupt
 * 3:    Not Used
 * 4:    I2C0 Interrupt
 * 5:	 Free
 * 6-16: Not Used
 */
int main (void)
{
	OSHANDLES SysHandles;        // Should contain all OS Handles

	cpuSetupHardware();          // Setup PLL, enable MAM etc.
	watchdogDelayUs(1000*1000);  // Some startup delay
	uart0Init(38400, 128);       // 128 is size of Rx/Tx FIFO

	// Use polling version of uart0 to do printf/rprintf before starting FreeRTOS
	rprintf_devopen(uart0PutCharPolling);
	cpuPrintMemoryInfo();

	// Open interrupt-driven version of UART0 Rx/Tx
	rprintf_devopen(uart0PutChar);
	watchdogDelayUs(1000*1000);

	Timer diskTimer(FatFsDiskTimer, 10, TimerPeriodic);
	diskTimer.start();

	//Create Mutexs and Queues for communicating between tasks/devices
	SysHandles.lock.SPI = xSemaphoreCreateMutex();
	SysHandles.queue.mp3CTRL = xQueueCreate(2, 2);
	SysHandles.queue.trackCTRL = xQueueCreate(1, sizeof(int));
	SysHandles.queue.songNameQueue = xQueueCreate(13, 13);
	SysHandles.queue.ctrlToSD = xQueueCreate(1, sizeof(int));

	if (
		pdPASS != xTaskCreate( uartUI, (signed char*)"Uart UI", STACK_BYTES(1024*6), &SysHandles, PRIORITY_LOW,  &SysHandles.task.userInterface )
		||
		pdPASS != xTaskCreate( mp3Task, (signed char*)"MP3", STACK_BYTES(1024*6), &SysHandles, PRIORITY_HIGH,  &SysHandles.task.mp3 )
		||
		pdPASS != xTaskCreate( sdTask, (signed char*)"SDtask", STACK_BYTES(1024*2), &SysHandles, PRIORITY_LOW, &SysHandles.task.SD_card )
		||
		pdPASS != xTaskCreate( buttonTASK, (signed char*)"Button Monitor", STACK_BYTES(1024), &SysHandles, PRIORITY_LOW, &SysHandles.task.buttonMonitor )
		||
		pdPASS != xTaskCreate( tsopDecoderTask, (signed char*)"TSOP1236 Task", STACK_BYTES(2048), &SysHandles, PRIORITY_HIGH, &SysHandles.task.tsop1236Task)
	){
		rprintf("ERROR:  OUT OF MEMORY: Check OS Stack Size or task stack size.\n");
	}
	// Start FreeRTOS to begin servicing tasks created above, vTaskStartScheduler() will not "return"
	rprintf("\n-- Starting FreeRTOS --\n");
	vTaskStartScheduler();

	// In case OS is exited:
	rprintf_devopen(uart0PutCharPolling);
	rprintf("ERROR: Unexpected OS Exit!\n");

	return 0;
}

/********************************************************************************
 * TODO 7.  Hints (Please read)
 *
 *  I'm totally lost!
 *  	Start from TODOs, and that should enable you to play ONE song.
 *  	Get one song to play, the rest of the pieces will fall in place.
 *      Read this documention on how to open and read a file:
 *      	http://elm-chan.org/fsw/ff/00index_e.html
 *
 *	How do I get listing of MP3 files on SD Card?
 *		- Reference "ls" command at uartUI, it does file listing.  You need to
 *			reuse this code, and modify it so that you can use it to scan for
 *			MP3 files in your SD Card root folder.
 *			Don't get scared of the code, remove iprintf, and see how simple
 *			the code is.  File name is at this variable: Finfo.fname[0]
 *
 *  Ok, I'm able to read a file, how exactly do I play it?
 *			Once you read a file and store to buffer, such as "char buff[512]"
 *			you simply need to "Hand it off" to MP3 Decoder called STA013
 *			This macro tells you when MP3 needs data: STA013_NEEDS_DATA()
 *			Once you detect that MP3 decoder needs data, simply transfer it
 *			using your SPI transfer function.
 *
 * *****************************************************************************/
